/*
 * This program is Copyright 1998 by Henry Cohen commercial rights are reserved.
 * The program is provided as is with no warranties of any sort including 
 * fitness for any purpose.  In no case will the author be held liable for
 * damages related to the use of this program.

 * The author hereby grants rights to use, copy, modify and create derived
 * products for individual, non-commercial purposes provided a copy of this
 * notice is included.  Any commercial use requires the written approval of 
 * the author.
 * Henry Cohen 1998.
 */

package jdic;
import java.awt.*;
import java.awt.event.*;


/**
 * class Gui - This class implements the graphic user interface.
 * The gui is built in a frame with some menus.  The menus are all built
 * by the method makeMenus() and are described in detail there.
 * Next there is a textArea for data input.  This is where the user enters
 * words to be looked up.  On the same line as the input area are two buttons
 * one for kanji entry which will bring up dialogs for entering kanji by
 * various methods, initially bushu and stroke but later perhaps also skip.
 * I will probably eventually support entry of various codes such as Unicode, 
 * SJIS, JIS, EUC, Kuten etc via this dialog.  I certainly want to do at least
 * bushu and components.
 * The last button calls up Kdict.  I expect Kdict to work on either a selection
 * in either the input or output text areas or on whatever is in the input area.
 * Next occupying most of the screen area is an output field that holds the result
 * of searches.  The user can make selections in this text area but cannot edit
 * it.  Finally the bottom line of the screen is a status line to give some 
 * indication of what the dictionary is doing from time to time.
 */
class Gui extends Frame{
    Gui guiFrame;
    private MyTextField guiInputField;
    private Button guiKinputButton, guiKdictButton;
    private MyTextArea guiOutputTextArea;
    private Label guiStatusBarStatic;
    Label guiStatusBar;
    public boolean firstKanji = true;  // true if matches must be first kanji.
    private KanjiInput kInput;
    
   
    /**
     * constructor Gui() constructor of the Gui class.  It is expected
     * that this constructor will be called only once although we do not
     * use any static fields or methods so I suppose we could do things like
     * have multiple gui'S for different dictionaries.  Initially I expect to
     * support only one gui for all dictionaries.  Some of the GridBagConstraints
     * parameters may seem a bit mysterious (they certainly do to me) anyway
     * they are the way that they are because that is what they need to be to
     * get the appearance and resizing behavior that I want.  I can't explain 
     * them any better than that. 
     */
    public Gui()  {
	super("Japanese/English Dictionary");
	guiFrame = this;
	GridBagConstraints  constraints = new GridBagConstraints();
	
	addWindowListener( new GuiWindowListener());
	setLayout(new GridBagLayout());
	
	guiInputField = new MyTextField("",30);
	guiInputField.addActionListener(new ActionListener(){
	    public void actionPerformed(ActionEvent e) {
		JJDic.formQuery(e.getActionCommand());
	    }
	});
	constraints.fill = GridBagConstraints.HORIZONTAL;
	constraints.weightx = 1;
	constraints.anchor = GridBagConstraints.NORTHWEST;
	constraints.gridwidth = 1;
	add(guiInputField, constraints);

	Checkbox cb = new Checkbox("\u63a5\u982d",true);
	cb.addItemListener(new ItemListener (){
	    public void itemStateChanged (ItemEvent e) {
		guiFrame.firstKanji = ((Checkbox)e.getSource()).getState();
	    }
	});
	constraints.weightx = 0;
	constraints.gridwidth = 1;
	constraints.fill = GridBagConstraints.BOTH;
       	add(cb, constraints);

	guiKinputButton = new Button("\u6F22\u5b57\u5165\u529B");
	guiKinputButton.setEnabled(false);
	guiKinputButton.addActionListener(new GuiKinbuttonListener());
	constraints.weightx = 0;
	constraints.gridwidth = GridBagConstraints.RELATIVE;
	constraints.fill = GridBagConstraints.BOTH;
	add(guiKinputButton, constraints);

	guiKdictButton = new Button("\u6F22\u5b57\u8f9e\u5178");
	guiKdictButton.setEnabled(false);
	guiKdictButton.addActionListener(new ActionListener (){
	    public void actionPerformed(ActionEvent e) {
		String oa_selection = guiOutputTextArea.getSelectedText();
                if (oa_selection.length() > 0) JJDic.kdictQuery(oa_selection);
		else {
		    String ia_selection = guiInputField.getSelectedText();
		    if (ia_selection.length() > 0) JJDic.kdictQuery(ia_selection);
		    else {
			String ia_text = guiInputField.getText();
			if (ia_text.length() > 0) JJDic.kdictQuery(ia_text);
		    } // else nothing selected so do nothing
		}
	    }
	});
	constraints.weightx = 0;
	constraints.gridwidth = GridBagConstraints.REMAINDER;
	constraints.fill = GridBagConstraints.BOTH;
	add(guiKdictButton, constraints);

	guiOutputTextArea = new MyTextArea("",24,80,TextArea.SCROLLBARS_BOTH);
	guiOutputTextArea.setEditable(false);
	
	constraints.fill = GridBagConstraints.BOTH;
	constraints.weightx = 1;
	constraints.weighty = 1;
	constraints.gridwidth =  GridBagConstraints.REMAINDER;
	add(guiOutputTextArea, constraints);

	constraints.fill = GridBagConstraints.NONE;
	constraints.anchor = GridBagConstraints.EAST;
	constraints.weightx = 0;
	constraints.weighty = 0;   
	guiStatusBarStatic = new Label("Status: ");
	constraints.gridwidth =  1;
	add(guiStatusBarStatic, constraints);
	
	constraints.fill = GridBagConstraints.HORIZONTAL;
	constraints.anchor = GridBagConstraints.WEST;
	constraints.weightx = 2;
	constraints.weighty = 0;   
	guiStatusBar = new Label("Initial");
	constraints.gridwidth =  GridBagConstraints.REMAINDER;
	add(guiStatusBar, constraints);
	
	makeMenus();
	pack();
	show();	
	/*
	 * At least in Windoze the checkbox background is the same as it's container
	 * while the button background is grey.  This looks terrible so kludge it
	 * into something reasonable.
	 */	
	cb.setBackground(guiKinputButton.getBackground());
    }
    /**
     * method makeMenus(frame) makes up a menu bar and adds to frame.
     *
     * The menu bar has the following menus and items
     * File          Dictionaries    Filters           Options         Help
     *   CloseDict(C)   <One check       Standard(F)     Fonts       Version(V)
     *                   box menu        User (U)        ShowDict    Help(H)
     *   Exit(X)        entry for                      
     *                  each open dict>
     *
     * Menus items have the indicated keyboard shortcuts but I ran out
     * of good mnemonic letters so the Fonts menu item does not have a
     * shortcut.  
     *
     * [Soap] Although it doesn't say so anywhere in the Java documentation
     * it seems to make a difference what order you use for menu building.
     * My first choice was to build bottom up, i.e. make the menus and items
     * and then add them to the menu bar and finally add the fuklly constructed
     * menu bar to the Frame.  This doesn't work.  Perhaps it's a bug.
     * What works is to build from the top down. ie. make a menu bar and
     * attach it to the frame, then make the menus and attach them to the
     * menu bar, finally make the menu items and attach them to the menus.
     * Another comment is that the menu shortcuts supported by Java seem to
     * only be able to directly access leaf node "MenuItems".  They do not
     * appear to support the kind of tree traversal of the menu tree that 
     * you can do in Windoze or Motif.  You can attach a shortcut to a menu
     * but it doesn't seem to have any descernable effect.
     */
    MenuBar guiMenuBar;
    Menu guiFileMenu, guiFiltersMenu, guiOptionsMenu, guiDictionariesMenu, guiHelpMenu;
    MenuItem guiFileOpenItem,  guiFilePrintItem, guiFileExitItem;
    MenuItem guiFiltersStandard, guiFiltersCustom;
    MenuItem guiOptionFonts;
    CheckboxMenuItem guiOptionShow;
    MenuItem guiHelpItem;
        
    private void makeMenus() {
	guiMenuBar = new MenuBar();
	this.setMenuBar(guiMenuBar);
	/*
	 * File menu
	 */

	guiFileMenu = new Menu("File");
	guiMenuBar.add(guiFileMenu);

	guiFileOpenItem = new MenuItem("Open Dict", new MenuShortcut('O'));
	guiFileOpenItem.addActionListener(new MenuOpenListener());
	//	guiFileOpenItem.setEnabled(false);
	guiFileMenu.add(guiFileOpenItem);

	guiFilePrintItem = new MenuItem("Print", new MenuShortcut('P'));
	guiFilePrintItem.addActionListener(new MenuPrintListener());
	guiFilePrintItem.setEnabled(false);
	guiFileMenu.add(guiFilePrintItem);
	
	guiFileExitItem = new MenuItem("Exit", new MenuShortcut('x'));
	guiFileExitItem.addActionListener(new MenuExitListener());
	guiFileMenu.add(guiFileExitItem);	
	/*
	 * Dictionaries menu
	 * Checkbox Menu Items are added to this menu as dictionaries are loaded.
	 * If the check box is checked (the default) then the dictionary will
	 * be searched.  If not checked then it will not be searched.  If 
	 * a dictionary is closed then it's entry will be removed from the 
	 * menu.
	 */
	guiDictionariesMenu = new Menu("Dictionaries");
	guiMenuBar.add(guiDictionariesMenu);
	
	/*
	 * Filters menu
	 */
	guiFiltersMenu = new Menu("Filters");
	guiMenuBar.add(guiFiltersMenu);
	
	guiFiltersStandard = new MenuItem("Standard",new MenuShortcut('F'));
	guiFiltersStandard.addActionListener(new MenuStdFiltListener());
	guiFiltersStandard.setEnabled(false);
	guiFiltersMenu.add(guiFiltersStandard);
	
	guiFiltersCustom = new MenuItem("User",new MenuShortcut('U'));
	guiFiltersCustom.addActionListener(new MenuCustomFiltListener());
	guiFiltersCustom.setEnabled(false);
	guiFiltersMenu.add(guiFiltersCustom);
	
	/*
	 * Options menu
	 */
	guiOptionsMenu = new Menu("Options");
	guiMenuBar.add(guiOptionsMenu);

	guiOptionFonts = new MenuItem("Fonts", new MenuShortcut('N'));
	guiOptionFonts.addActionListener(new MenuFontListener());
	guiOptionsMenu.add(guiOptionFonts);
	
	guiOptionShow = new CheckboxMenuItem("ShowDic",true);
	guiOptionShow.addItemListener( new ItemListener() {
	    public void itemStateChanged(ItemEvent e) {
		CheckboxMenuItem cb = (CheckboxMenuItem) e.getSource();
		JJDic.setShowDictNames(cb.getState());
	    }
	});
	guiOptionsMenu.add(guiOptionShow);
	
	/*
	 * Help menu
	 */
	guiHelpMenu = new Menu("Help");
	guiMenuBar.add(guiHelpMenu);
	guiMenuBar.setHelpMenu(guiHelpMenu);
	

	guiHelpItem = new MenuItem("Help",new MenuShortcut('H'));	
	guiHelpItem.addActionListener(new MenuHelpListener());
	guiHelpItem.setEnabled(false);
	guiHelpMenu.add(guiHelpItem);
		
    }
	
    void clearResults() {
	guiOutputTextArea.setText("");
    }
    void appendResults(String[] results) {
	if (results != null) {
	    for (int i = 0; i< results.length; i++) {
		guiOutputTextArea.append(results[i]);
	    }
	}
    }
    void setGuiFont(Font guiFont) {
	guiOutputTextArea.setFont(guiFont);
	guiOutputTextArea.repaint();
	guiInputField.setFont(guiFont);
	guiInputField.repaint();
    }
    void kdictLoaded(){
	guiKdictButton.setEnabled(true);
	guiKinputButton.setEnabled(true);
    }
    /*
     * Open Dictionary Menu related functions and inner classes.
     */
    /**
     * inner class MenuOpenListener handles action events from the 
     * open menu.
     * Actions relating to the GUI are done here.  Actions relating to
     * dictionary loading and verification are done by the JJDic class.
     * The method addDictMenu() is called from the JJDic class after the
     * dictionary has been loaded to add an entry to the dictionaris menu.
     */
    private class MenuOpenListener implements ActionListener{
	/*
	 * Open a new dictionary.  If no dictionaries are found in the
	 * user's properties file then even edict and kdict must be opened
	 * this way.
	 * Put up a dialog to get the file, encoding and createIndex boolean.
	 * Only the buttons int this dialog need action listeners.  Everything
	 * else can be read back at the end.
	 */
	MyTextField fileField;
	String directory, filename;
	boolean cancel = false;
	Dialog openDialog;
			
	public void actionPerformed(ActionEvent e) {

	    openDialog = new Dialog(guiFrame, "Open Dictionary",true);
	    openDialog.addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent e){
		    cancel = true;
		    openDialog.setVisible(false);
		    guiFrame.show();
		}
	    });
	    openDialog.setLayout(new GridBagLayout());
	    GridBagConstraints cstr = new GridBagConstraints();
	    
	    fileField = new MyTextField(30);
	    fileField.addActionListener(new ActionListener() {
		public void actionPerformed (ActionEvent e) {
		    cancel = false;
		    openDialog.setVisible(false);
		}
	    }
					);
	    cstr.fill = GridBagConstraints.HORIZONTAL;
	    cstr.gridx = 0;
	    cstr.gridy = 0;
	    cstr.weightx = 1;
	    cstr.gridwidth = 2;
	    openDialog.add(fileField, cstr);
	    
	    Button searchButton = new Button("Search");
   	    searchButton.addActionListener(new SearchButtonListener());
	    cstr.fill = GridBagConstraints.NONE;
	    cstr.gridx = 2;
	    cstr.gridy = 0;
	    cstr.weightx = 0;
	    cstr.gridwidth = 1;
	    cstr.anchor = GridBagConstraints.WEST;
	    openDialog.add(searchButton, cstr);
	    
	    Label encodingLabel = new Label("Encoding", Label.RIGHT);
	    cstr.fill = GridBagConstraints.NONE;
	    cstr.gridx = 0;
	    cstr.gridy = 1;
	    cstr.insets.right = 4;
	    cstr.weightx = 0;
	    cstr.anchor = GridBagConstraints.EAST;
	    openDialog.add(encodingLabel, cstr);
	    cstr.insets.left = 0;
	    
	    CheckboxGroup encodingCBG = new CheckboxGroup();
	    Checkbox eucCB = new Checkbox("EUCJIS",encodingCBG, true);
	    cstr.fill = GridBagConstraints.HORIZONTAL;
	    cstr.gridx = 1;
	    cstr.gridy = 1;
	    cstr.anchor = GridBagConstraints.WEST;
	    openDialog.add(eucCB, cstr);
	    
	    Checkbox jisCB = new Checkbox("JIS",encodingCBG, false);
	    cstr.fill = GridBagConstraints.HORIZONTAL;
	    cstr.gridx = 1;
	    cstr.gridy = 2;
	    cstr.anchor = GridBagConstraints.NORTH;
	    openDialog.add(jisCB, cstr);

	    Checkbox sjisCB = new Checkbox("SJIS",encodingCBG, false);
	    cstr.fill = GridBagConstraints.HORIZONTAL;
	    cstr.gridx = 1;
	    cstr.gridy = 3;
	    cstr.anchor = GridBagConstraints.NORTH;
	    openDialog.add(sjisCB, cstr);

	    /*
	     * NB Not a part of the previous CheckboxGroup
	     */
	    Checkbox indexCB = new Checkbox("Create Index", true);
	    cstr.fill = GridBagConstraints.NONE;
	    cstr.gridx = 0;
	    cstr.gridy = 4;
	    cstr.anchor = GridBagConstraints.EAST;
	    openDialog.add(indexCB, cstr);

	    Button okButton = new Button("OK");
 	    okButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    cancel = false;
		    openDialog.setVisible(false);
		    guiFrame.show();
		}
	    });
	    cstr.gridx = 0;
	    cstr.gridy = 5;
	    cstr.insets.right = 4;
	    cstr.anchor = GridBagConstraints.EAST;
	    openDialog.add(okButton, cstr);
	    cstr.insets.right = 0;

	    Button canButton = new Button("Cancel");
 	    canButton.addActionListener(new ActionListener (){
		public void actionPerformed(ActionEvent e) {
		    cancel = true;
		    openDialog.setVisible(false);
		    guiFrame.show();
		}
	    }
					);
	    cstr.gridx = 1;
	    cstr.gridy = 5;
	    cstr.insets.left = 4;
	    cstr.anchor = GridBagConstraints.WEST;
	    openDialog.add(canButton, cstr);

	    openDialog.pack();
	    openDialog.show();
	    if (cancel) return;   // Did somebody push the cancel button?
	    String path = fileField.getText(); // Get the filename.
	    if (path == null) return; // Didn't set a filename so return
	    boolean createIndex = indexCB.getState();
	    String encoding = encodingCBG.getSelectedCheckbox().getLabel();
	    openDialog.dispose();
	    	  
   	    JJDic.loadNewDictionary(path, encoding, createIndex);
	}

	
	class SearchButtonListener implements ActionListener {
	    public void actionPerformed(ActionEvent e) {
		String path;
	     
		FileDialog fd = new FileDialog(guiFrame, "Open Dictionary File");
		fd.show();
		directory = fd.getDirectory();
		filename =  fd.getFile();
		fd.dispose();	    
		if(filename != null) {
		    path = filename;
		    if (directory != null) {
			path = directory + System.getProperty("file.separator") + path;
		    }		 
		} else {
		    // No file selected
		    return;
		}
		fileField.setText(path);
	    }
	}
    }
    
    /**
     * method addDictMenu(String dname) add a menu item to the dictionary menu
     * labled dname.
     * dname is presumably the name of the dictionary.
     * the item is a CheckboxMenuItem and it is initially checked.
     */
    String addDictMenu(String dname) {
	/*
	 * strip off any leading path components leaving only the filename
	 */
	int fsep = dname.lastIndexOf(System.getProperty("file.separator").charAt(0));
	if (fsep > 0) dname = dname.substring(fsep+1);
	
	/*
	 * Strip of any suffix <name>.suffix name can begin with '.'
	 * The reason for this is that I want to differentiate duplicate
	 * names as name.1 name.2.  Stripping off other .suffix trash will
	 * make it a lot easier.
	 */
	int idx = dname.indexOf('.');
	if (idx>0) {   
	    dname = dname.substring(0,idx);
	}
	
	/*
	 * First check to see that nothing already has the same name.
	 * If there is another such item then change the name to have
	 * a slightly different form.  I scan the menu list bottom to top
	 * so that the first match will be the last entered.  If it has a 
	 * .
	 */
	MenuItem item;
	String label, suffix;
	
	for (int i = guiDictionariesMenu.getItemCount()-1; i >= 0;  i--){
	    item = guiDictionariesMenu.getItem(i);
	    label = item.getLabel();
	    idx = label.indexOf('.');

	    if (idx>0) {   // strip off .suffix and compute a new one
		suffix = label.substring(idx+1);
		label = label.substring(0,idx);
		try{
		    suffix = "."+ // Convert string to int add 1 and convert back.
			String.valueOf(Integer.valueOf(suffix).intValue()+1);
		} catch (NumberFormatException e){
		    suffix = ".1";  // Shoudn't get this but if we do ...
		}
	    } else {
		suffix = ".1";
	    }
	    if (dname.equals(label)){
		dname = dname+suffix;
		break;
	    }
	}
	/*
	 * Now add the new name to the dictionaries menu
	 */
	CheckboxMenuItem mi = new CheckboxMenuItem(dname, true);
	mi.addItemListener(new ItemListener(){
	    public void itemStateChanged(ItemEvent e) {
		CheckboxMenuItem cb = (CheckboxMenuItem) e.getSource();
		JJDic.setDictionaryState(cb.getLabel(), cb.getState());
	    }
	});
	guiDictionariesMenu.add(mi);
	guiFrame.validate();
	guiFrame.repaint();
	return dname;
    }

    
    /*
     * Event Listener classes for Window events
     */
    private class GuiWindowListener extends WindowAdapter{
	public void windowClosed(WindowEvent e) {
	    System.exit(1);
	}
	public void windowClosing(WindowEvent e) {
	    System.exit(1);
	}
    }
    /*
     * Event listeners for menu selection Action Events.
     * If a listener only requires a few lines of code then use
     * an anonymous inner class at the point of use.
     */
    class MenuPrintListener implements ActionListener{
	public void actionPerformed(ActionEvent e) {
	}
    }
    private class MenuExitListener implements ActionListener{
	public void actionPerformed(ActionEvent e) {
	    System.exit(1);
	}
    }
    private class  MenuStdFiltListener implements ActionListener{
	public void actionPerformed(ActionEvent e) {
	}
    }
    private class MenuCustomFiltListener implements ActionListener{
	public void actionPerformed(ActionEvent e) {
	}
    }
    /*
     * Font Menu Item Action Listener.
     * Put up a font selection dialog and call
     * jdic.Gui.setGuiFont() with the result.
     * This selects a font for the Input text field and output text area only.
     * The fonts for other GUI components are allowed to default.  The reason
     * for this is that Java requires a size selection when fonts are selected.
     * I want the fonts for Labels and Buttons to be automatically scaled to the 
     * size of the component so I will let the AWT do it.
     */
    private class MenuFontListener implements ActionListener{
	private Dialog fontDialog;
	Choice fontChoice, styleChoice, sizeChoice;
	
	public void actionPerformed(ActionEvent e) {
	    Button okButton, cancelButton;
	    int style, size;
	    Font curFont;
	    
	    String[] fontNameList; //, fontFamilyList;
//	    Font[] availableFonts;
// 	    int Nfonts = 0;	    

	    /*
	     * This next line is almost platform dependent.  It should still
	     * be 100% pure Java though but they might object to accessing
	     * the Toolkit directly.
	     * fontlist is the System defined list of font names.
	     * fontNameList is sorted with redundant elements removed.
	     * [Soap] I don't like the fact that I have to get font
	     * objects at this point because it seems wasteful of memory
	     * and time but this is not an inner loop so the time is not
	     * critical and the memory is probably minimal.  It is nontheless
	     * unnecessary but I can't get a Font family name without
	     * creating an instance of the font. More soap.  C structure
	     * assignment would take care of this much more neatly.
	     * [Soap] This would be nice if it worked! Unfortunately we are
	     * screwed by portability concerns or something again.  The
	     * Font method getFamily doesn't really return the platform
	     * specific font name after all so we are stuck with using
	     * the Java virtual names.
	     */
	    fontNameList = Toolkit.getDefaultToolkit().getFontList();

	    /*
	     * Get family names, sort and eliminate duplications.
	     * I have submitted a bug report.  If Sun thinks it's a bug
	     * they may fix it and this may become useful but in the
	     * meantime punt.
	     */
// 	    availableFonts = new Font[fontNamelist.length];
// 	    fontFamilyNames = new String[fontNamelist.length];
	    
// 	    for (int i = 0; i< fontNamelist.length; j++) {
// 		availableFonts[i] = new Font(fontNamelist[i], Font.PLAIN, 12);
// 		fontFamilyNames[i] = availableFonts[i].getFamily();
// 	    }
// 	    String K, Fn; Font F;
// 	  j:for (int j = 1; j< fontNameList.length; j++) {
// 	        for (int i = j-1, K = fontFaimilyNames[j], F = availableFonts[j],
// 			 Fn = fontNameList[j]; i>=0; i--) {
// 		    if (K.compareTo(fontFamilyNames[i])>=0){
// 			fontFamilyNames[i+1] = K;
// 			fontNameList[i+1] = Fn;
// 			availableFonts[i+1] = F;
// 			continue j;
// 		    }
// 		    fontFamilyNames[i+1] = fontFamilyNames[i];
// 		    fontNameList[i+1] = fontNameList[i];
// 		    availableFonts[i+1] = availableFonts[i];
// 		}
// 		fontFamilyNames[0] = K;
// 		fontNameList[0] = Fn;
// 		availableFonts[0] = F;
// 	    }
// 	  /*
// 	   * Eliminate duplications
// 	   */
// 	  for (int i = 1; i < fontlist.length; i++) {
// 	      if (fontFamilyNames[i].compareTo(fontFamilyNames[Nfonts]) != 0) {
// 		  fontFamilyNames[++Nfonts] = fontFamilyNames[i];
// 		  fontNameList[Nfonts] = fontNameList[i];
// 		  availableFonts[Nfonts] = availableFonts[i];
// 	      }
// 	  }
	  /*
	   * Now we have a list of available fonts Let's ask the user
	   * what she wants to use.
	   */
	  curFont = guiInputField.getFont();

	  fontDialog = new Dialog(guiFrame, "Select Font", true);
	  fontDialog.setLayout(new FlowLayout());
	  fontChoice = new Choice();
	  for (int i = 0; i<fontNameList.length; i++) {
	      fontChoice.addItem(fontNameList[i]);
	  }
	  fontChoice.select(curFont.getName());
	  fontDialog.add(fontChoice);
	  
	  styleChoice = new Choice();
	  styleChoice.addItem("Plain");	  
	  styleChoice.addItem("Bold");
	  styleChoice.addItem("Italic");
	  if (curFont.isPlain()) 
	      styleChoice.select("Plain");
	  else if (curFont.isBold())
	      styleChoice.select("Bold");
	  else if (curFont.isItalic())
	      styleChoice.select("Italic");
	  fontDialog.add(styleChoice);
	  
	  size = curFont.getSize();
	  sizeChoice = new Choice();
	  sizeChoice.addItem("10");
	  sizeChoice.addItem("12");
	  sizeChoice.addItem("14");
	  sizeChoice.addItem("18");
	  sizeChoice.addItem("20");	  
	  if (size <= 10)
	     sizeChoice.select("10");
	  else if (size <= 12)
	     sizeChoice.select("12");
	  else if (size <= 14)   
	     sizeChoice.select("14");
	  else if (size <= 18)
	     sizeChoice.select("18");
	  else 
	     sizeChoice.select("20");
	  fontDialog.add(sizeChoice);
	  
	  okButton = new Button("OK");
	  okButton.addActionListener(new OKListener());
	  fontDialog.add(okButton);
	  
	  cancelButton = new Button("Cancel");
	  cancelButton.addActionListener(new ActionListener() {
	      public void actionPerformed(ActionEvent e) {
		  fontDialog.dispose();
		  guiFrame.show();
	      }
	  });
	  fontDialog.add(cancelButton);
	  fontDialog.pack();
	  fontDialog.setVisible(true);
	}
	private class OKListener implements ActionListener {
	    public void actionPerformed(ActionEvent e){
		String style; int istyle;
		Font newFont;
		
		fontDialog.setVisible(false);
		style = styleChoice.getSelectedItem();
		if (style.equals("Plain")){
		    istyle =  Font.PLAIN;
		} else 	if (style.equals("Bold")){
		    istyle =  Font.BOLD;
		} else  if (style.equals("Italic")){
		    istyle =  Font.ITALIC;
		} else {
		    istyle = Font.PLAIN;
		}
		newFont = new Font(fontChoice.getSelectedItem(), istyle,
				    Integer.parseInt(sizeChoice.getSelectedItem()));
		guiFrame.setGuiFont(newFont);
		fontDialog.dispose();
		guiFrame.show();
	    }
	}
    }

    private class MenuHelpListener implements ActionListener{
	public void actionPerformed(ActionEvent e) {
	}
    }
    /*
     * Event listeners for events associated with other GUI objects.
     */
    private class GuiKinbuttonListener implements ActionListener{
	public void actionPerformed(ActionEvent e) {
	    if (kInput != null) {
		kInput.dispose();
	    } 
	    kInput = new KanjiInput(guiFrame, guiInputField);
	}
    }

    /**
     * package method setStatus(String s) write s in status area.
     * This method has package visibility so that any part of the 
     * application can write status messages.
     */
    void setStatus(String s){
	guiStatusBar.setText(s);
	this.validate();
	this.repaint();
    }
    String getKanjiPath() {
	String path, directory, filename;
	     
	FileDialog fd = new FileDialog(guiFrame, "Where is Kanjidic?");
	fd.show();
	directory = fd.getDirectory();
	filename =  fd.getFile();
    	fd.dispose();
	if(filename != null) {
	    path = filename;
	    if (directory != null) {
		path = directory + System.getProperty("file.separator") + path;
	    }		 
	} else {
	    // No file selected
	    return null;
	}
	return path;
    }
    /*
     * The method kanjiIndexDialog prepares a dialog for selecting among
     * several kanji from kanjidic.  When the user does a kanjidic lookup
     * the results are summarized in this panel and individual entries are
     * displayed by selecting items from the button panels.
     */
    Button onButton, kunButton, kanButton, eiButton;
    Panel cardPanel;
    boolean kanjiAri, onYomiAri, kunYomiAri, eigoAri;
    Dialog kindialog;
    
    void kanjiIndexDialog (String[]kanjiList, String[]onYomiList, String[]kunYomiList, String[]eigoList) {
	Panel  buttonPanel;
	HyoujiPanel kanjiPanel, onYomiPanel, kunYomiPanel, eigoPanel;
	kanjiAri = kanjiList != null;  
	onYomiAri = onYomiList != null; 
	kunYomiAri = kunYomiList != null; 
	eigoAri = eigoList != null;
	if (kindialog != null) kindialog.dispose();
	kindialog = new Dialog(guiFrame);
	kindialog.setLayout(new BorderLayout());
	kindialog.addWindowListener(new WindowAdapter(){
	    public void windowClosing(WindowEvent e) {
		kindialog.setVisible(false);
		kindialog.dispose();
		guiFrame.toFront();
	    }
	});

	buttonPanel = new Panel();
	onButton = new Button("\u97f3\u8aad"); // On Yomi
	onButton.setEnabled(onYomiAri);
	onButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e){
		    onButton.setEnabled(onYomiAri & false);
		    kunButton.setEnabled(kunYomiAri & true);
		    kanButton.setEnabled(kanjiAri & true);
		    eiButton.setEnabled(eigoAri & true);	    
		    ((CardLayout)cardPanel.getLayout()).show(cardPanel,"\u97f3\u8aad");
		    kindialog.setTitle("\u97f3\u8aad");
		}
	    });
	buttonPanel.add(onButton);

	kunButton = new Button("\u8a13\u8aad");  // Kun Yomi
	kunButton.setEnabled(kunYomiAri);
	kunButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e){
		    onButton.setEnabled(onYomiAri & true);
		    kunButton.setEnabled(kunYomiAri & false);
		    kanButton.setEnabled(kanjiAri & true);
		    eiButton.setEnabled(eigoAri & true);		    
		    ((CardLayout)cardPanel.getLayout()).show(cardPanel,"\u8a13\u8aad");
		    kindialog.setTitle("\u8a13\u8aad");
		}
	    });
	buttonPanel.add(kunButton);

	kanButton = new Button("\u6f22\u5b57");  // Kanji
	kanButton.setEnabled(kanjiAri);
	kanButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e){
		    onButton.setEnabled(onYomiAri & true);
		    kunButton.setEnabled(kunYomiAri & true);
		    kanButton.setEnabled(kanjiAri & false);
		    eiButton.setEnabled(eigoAri & true);		    
		    ((CardLayout)cardPanel.getLayout()).show(cardPanel,"\u6f22\u5b57");
		    kindialog.setTitle("\u6f22\u5b57");
		}
	    });
	buttonPanel.add(kanButton);

	eiButton = new Button("\u82f1\u8a9e");  // Eigo
	eiButton.setEnabled(eigoAri);
	eiButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e){
		    onButton.setEnabled(onYomiAri & true);
		    kunButton.setEnabled(kunYomiAri & true);
		    kanButton.setEnabled(kanjiAri & true);
		    eiButton.setEnabled(eigoAri & false);		    
		    ((CardLayout)cardPanel.getLayout()).show(cardPanel,"\u82f1\u8a9e");
		    kindialog.setTitle("\u82f1\u8a9e");
		}
	    });
	buttonPanel.add(eiButton);
	kindialog.add(buttonPanel, "North");
	
	kanjiPanel = new HyoujiPanel(kanjiList);
	onYomiPanel = new HyoujiPanel(onYomiList);
        kunYomiPanel = new HyoujiPanel(kunYomiList);
	eigoPanel = new HyoujiPanel(eigoList);

        cardPanel = new Panel();
	cardPanel.setLayout(new CardLayout());
	cardPanel.add(kanjiPanel,"\u6f22\u5b57");
	cardPanel.add(onYomiPanel,"\u97f3\u8aad");
	cardPanel.add(kunYomiPanel,"\u8a13\u8aad");
	cardPanel.add(eigoPanel,"\u82f1\u8a9e");
	if (kanjiAri){
	    ((CardLayout)cardPanel.getLayout()).show(cardPanel,"\u6f22\u5b57");    
	} else if (onYomiAri) {
	    ((CardLayout)cardPanel.getLayout()).show(cardPanel,"\u97f3\u8aad");
	} else if (kunYomiAri) {
	    ((CardLayout)cardPanel.getLayout()).show(cardPanel,"\u8a13\u8aad");
	} else if (eigoAri) {
	    ((CardLayout)cardPanel.getLayout()).show(cardPanel,"\u82f1\u8a9e");
	}
	kindialog.add(cardPanel,"Center");
	kindialog.pack();
	kindialog.setVisible(true);
    }
}
