/*
 * This program is Copyright 1998 by Henry Cohen commercial rights are reserved.
 * The program is provided as is with no warranties of any sort including 
 * fitness for any purpose.  In no case will the author be held liable for
 * damages related to the use of this program.

 * The author hereby grants rights to use, copy, modify and create derived
 * products for individual, non-commercial purposes provided a copy of this
 * notice is included.  Any commercial use requires the written approval of 
 * the author.
 * Henry Cohen 1998.
 */

package jdic;
import java.awt.*;

class Kanji extends Canvas {
    /*
     * Instance variables the kdict data record.
     */
    String kanji;  // String of length 1 for convienience
    String onYomi, kunYomi;
    String eigo;
    int eigoAdvance;
    String jisCode, nelson, halpern;
    String bushu, classic;
    String strokes;
    Dimension dim;
    /*
     * Static variables used for the display
     */
    static Font bannerFont, titleFont, dataFont;
    static FontMetrics bannerMetrics, titleMetrics, dataMetrics;
    static boolean initialized = false;
    static final String onLabel = "\u97f3\u8aad: ";
    static final String kunLabel = "\u8a13\u8aad: ";
    static final String bushuLabel = "\u90e8\u9996: ";
    static final String strokesLabel = "\u75E9\u753B\u6570: ";
    static final String jisLabel = "JIS: ";
    static final String uniLabel = "Unicode: ";
    static final String nelLabel = "Nelson: ";
    static final String halLabel = "Halpern: ";
    static final String eiLabel = "\u82f1\u8a9e: ";
    static final int BORDER = 4;    // general white space
    static final int BANNERPAD = 2; // padding to adjust the banner kanji within it's window
    
    
    /*
     * and their computed lengths. Obviously can't be final
     */
    static int onLabAdvance, kunLabAdvance, jisLabAdvance, uniLabAdvance;
    static int nelLabAdvance, halLabAdvance, eiLabAdvance;
    static int commaSpace, bushuLabAdvance, strokesLabAdvance;
    static int bannerSide;  // The banner kanji is drawn inside this square
    static int bannerBaseline, bannerOffset, line1Baseline, line2Baseline, line3Baseline;
    static int lineHeight;  // line spacing at 1.5 * font height.
    
    /*
     * class constructor takes a kanjidict entry as an argument.
     */    
    public Kanji(String kdictEntry) {
	int i, index;
	StringBuffer eigoBuffer = new StringBuffer();
	StringBuffer onBuffer = new StringBuffer();
	StringBuffer kunBuffer = new StringBuffer();
	StringBuffer strokesBuffer = new StringBuffer();
	char c;
	
	/*
	 * Parse the kanjidict string.
	 * The first entries are in fixed format
	 */
	kanji = kdictEntry.substring(0,1);
	jisCode = kdictEntry.substring(2, 6);
	/*
	 * after that they can be mixed up in any order.
	 * This is a standard regular expression scanner.
	 */

	try {
scan:	    
	    for (i = 7; i< kdictEntry.length(); i++){
		c = kdictEntry.charAt(i);
		while (c == ' ') c = kdictEntry.charAt(++i);
				
		if (c == 'N') {  // Nelson Number
		    c = kdictEntry.charAt(++i);
		    if (Character.isDigit(c)) {
			index = kdictEntry.indexOf(' ',i);
			nelson = kdictEntry.substring(i,index);
			i = index;
		    }
		} else if (c == 'H') {  // Halpern number
		    c = kdictEntry.charAt(++i);
		    if (Character.isDigit(c)) {
			index = kdictEntry.indexOf(' ',i);
			halpern = kdictEntry.substring(i,index);
			i = index;
		    }
		} else if (c == 'B') {
		    if (Character.isDigit(kdictEntry.charAt(++i))) {
			index = kdictEntry.indexOf(' ',i);
			bushu = kdictEntry.substring(i, index);
			i = index;
		    }
		} else if (c == 'S') {
		    if (Character.isDigit(kdictEntry.charAt(++i))) {
			index = kdictEntry.indexOf(' ',i);
			if (strokesBuffer.length() != 0) strokesBuffer.append(", ");
			strokesBuffer.append(kdictEntry.substring(i, index));
			i = index;
		    }
		} else if (c == 'C') {
		    if (Character.isDigit(kdictEntry.charAt(++i))) {
			index = kdictEntry.indexOf(' ',i);
			classic = kdictEntry.substring(i, index);
			i = index;
		    }
		} else if (c == '{') {
		    if (eigoBuffer.length() != 0) {
			eigoBuffer.append(", ");
		    } 
		    index = kdictEntry.indexOf('}',i);
		    eigoBuffer.append(kdictEntry.substring(++i, index));
		    i = index;
		} else if (MojiBunkai.isKatakana(c)) {
		    if (onBuffer.length() != 0)	onBuffer.append("; ");
		    if (kdictEntry.charAt(i-1) == '-') onBuffer.append('-');
		    onBuffer.append(c);
		    c = kdictEntry.charAt(++i);
		    while ((MojiBunkai.isKatakana(c))|| (c == '.') || (c == '-')) {
			onBuffer.append(c);
			c = kdictEntry.charAt(++i);
		    }
		} else if (MojiBunkai.isHiragana(c)) {
		    if (kunBuffer.length() != 0) kunBuffer.append("; ");
		    if (kdictEntry.charAt(i-1) == '-') kunBuffer.append('-');
		    kunBuffer.append(c);
		    c = kdictEntry.charAt(++i);
		    while (MojiBunkai.isHiragana(c)|| (c == '.') || (c == '-')) {
			kunBuffer.append(c);
			c = kdictEntry.charAt(++i);
		    }
		} else {
		    index = kdictEntry.indexOf(' ',i+1);
		    i = index;
		}
		if (i == -1) i = kdictEntry.length();
	    }
	} catch (StringIndexOutOfBoundsException e){
	    // No problem just eos.
	}
	if (eigoBuffer.length() != 0)
	    eigo = eigoBuffer.toString();
	if (onBuffer.length() != 0)
	    onYomi = onBuffer.toString();
	if (kunBuffer.length() != 0)
	    kunYomi = kunBuffer.toString();
	if (strokesBuffer.length() != 0)
	    strokes = strokesBuffer.toString();
    }

    /*
     * I assume that the component is layed out before it is painted.
     * If not then this won't work because paint assumes that a lot of
     * variables have already been initialized.
     * Initializations having to do with the object data are done in the
     * constructor, initializations having to do with the display are done
     * here.  It would be nice to do the graphic elements once as a class 
     * initialization but if this application is run as a client/server with
     * different clients on different platforms then the graphics information
     * may differ between clients.  The safest thing is to re-initialize for
     * each kanji display.
     */

    public Dimension getPreferredSize(){

	/*
	 * If no Dimension object yet exists then this is the first time so 
	 * allocate and initialize the Dimension object.  After that just return
	 * the precomputed value.
	 */
	if (dim == null) {
	    dim = new Dimension();
	    /*
	     * We need a bunch of fonts to work with along with their associated
	     * font metrics.  The fact that different platforms have different
	     * font metrics forces us to determine the metrics with respect to 
	     * the particular component instance.  This is why we cannot do this
	     * as a class initialization if we hope to support more than one kind 
	     * of client platform.
	     */
	    bannerFont = new Font("Serif", Font.BOLD, 36);
	    setFont(bannerFont);
	    bannerMetrics = this.getFontMetrics(bannerFont);
	    titleFont = new Font("SansSerif", Font.BOLD, 12);
	    setFont(titleFont);
	    titleMetrics = this.getFontMetrics(titleFont);
	    dataFont = new Font("SansSerif", Font.PLAIN, 12);
	    setFont(dataFont);
	    dataMetrics = this.getFontMetrics(dataFont);

	    /*
	     * Compute the lengths in pixels of the various static labels in
	     * our dialog.
	     */
	    onLabAdvance = titleMetrics.stringWidth(onLabel);
	    kunLabAdvance = titleMetrics.stringWidth(kunLabel);
	    jisLabAdvance = titleMetrics.stringWidth(jisLabel);
	    uniLabAdvance = titleMetrics.stringWidth(uniLabel);
	    nelLabAdvance = titleMetrics.stringWidth(nelLabel);
	    halLabAdvance = titleMetrics.stringWidth(halLabel);
	    bushuLabAdvance = titleMetrics.stringWidth(bushuLabel);
	    strokesLabAdvance = titleMetrics.stringWidth(strokesLabel); 
	    eiLabAdvance = titleMetrics.stringWidth(eiLabel); 
	    bannerSide = (int)(((float)bannerMetrics.charWidth('\u5b57')) * 1.1);
	    /*
	     * Compute text baselines within the dialog
	     */
	    bannerBaseline = BORDER + BANNERPAD + bannerMetrics.getAscent();
	    bannerOffset = BORDER + BANNERPAD;
	    lineHeight = Math.max(titleMetrics.getHeight(),dataMetrics.getHeight());
	    lineHeight  = (int)(1.5*lineHeight);
	    line1Baseline = BORDER + bannerSide/3;
	    line2Baseline = line1Baseline + lineHeight;
	    line3Baseline = BORDER + bannerSide + lineHeight;
	    // Other lines are spaced with lineHeight seperation.
	    /*
	     * The height is fixed and the width will vary depending on how long
	     * the various component strings are.
	     */
	    int lineWidth;
	    
	    dim.height = 2*BORDER + bannerSide + 4*lineHeight;
	    // Line 1
	    dim.width = 3*BORDER + bannerSide + onLabAdvance + 
		((onYomi != null)?dataMetrics.stringWidth(onYomi):0); 
	    // Line 2
	    lineWidth = 3*BORDER + bannerSide + kunLabAdvance + 
		((kunYomi != null)?dataMetrics.stringWidth(kunYomi):0); 
	    dim.width = Math.max(dim.width, lineWidth);
	    // Line 3
	    lineWidth = 3*BORDER + jisLabAdvance + uniLabAdvance +
		dataMetrics.stringWidth(jisCode) +
		dataMetrics.stringWidth(Integer.toHexString(kanji.charAt(0))); 
	    dim.width = Math.max(dim.width, lineWidth);
	    // Line 4
	    lineWidth = 3*BORDER + bushuLabAdvance + strokesLabAdvance +
		dataMetrics.stringWidth(bushu) + 
		dataMetrics.stringWidth(strokes); 
	    dim.width = Math.max(dim.width, lineWidth);
	    // Line 5
	    lineWidth = 3*BORDER + nelLabAdvance + halLabAdvance +
		((nelson != null)?dataMetrics.stringWidth(nelson):0) +
		((halpern != null)?dataMetrics.stringWidth(halpern):0); 
	    dim.width = Math.max(dim.width, lineWidth);
	    // Line 6
	    lineWidth = 2*BORDER + eiLabAdvance + 
		((eigo != null)?dataMetrics.stringWidth(eigo):0);
	    dim.width = Math.max(dim.width, lineWidth);	    
	} 
	return dim;
    }
    public Dimension getMinimumSize() {
	return getPreferredSize();
    }
    public void paint(Graphics g) {
	int advanceIncrement, point, baseline;
	/*
	 * Banner Kanji
	 */
	g.setColor(this.getBackground());
	g.fill3DRect(BORDER,BORDER, bannerSide, bannerSide, true);
	g.setColor(this.getForeground());
	g.setFont(bannerFont);
	g.drawString(kanji, bannerOffset, bannerBaseline);
	/*
	 * Line 1 On Yomi
	 */
	if (onYomi != null) {
	    g.setFont(titleFont);
	    point =  2*BORDER+bannerSide;
	    g.drawString(onLabel, point, line1Baseline);
	    point += onLabAdvance;
	    g.setFont(dataFont);
	    g.drawString(onYomi, point, line1Baseline);
	}
	/*
	 * Line 2 Kun Yomi
	 */
	if (kunYomi != null) {
	    point =  2*BORDER+bannerSide;
	    g.setFont(titleFont);
	    g.drawString(kunLabel, point, line2Baseline);
	    point += kunLabAdvance;
	    g.setFont(dataFont);
	    g.drawString(kunYomi, point, line2Baseline);
	}
	/*
	 * Line 3 JIS and Unicode
	 */
	point = BORDER;
	g.setFont(titleFont);
	g.drawString(jisLabel, point, line3Baseline);
	point += jisLabAdvance;
	g.setFont(dataFont);
	g.drawString(jisCode, point, line3Baseline);
	point += dataMetrics.stringWidth(jisCode) + BORDER;
	g.setFont(titleFont);
	g.drawString(uniLabel, point, line3Baseline);
	point += uniLabAdvance;
	g.setFont(dataFont);
	g.drawString(Integer.toHexString(kanji.charAt(0)), point, line3Baseline);
	
	/*
	 * Line 4 Bushu & Strokes
	 */
	baseline = line3Baseline+lineHeight;
	point = BORDER;
	g.setFont(titleFont);
	g.drawString(bushuLabel, point, baseline);
	point += bushuLabAdvance;
	g.setFont(dataFont);
	g.drawString(bushu, point, baseline);
	point += dataMetrics.stringWidth(bushu) + BORDER;

	g.setFont(titleFont);
	g.drawString(strokesLabel, point, baseline);
	point += strokesLabAdvance;
	g.setFont(dataFont);
	g.drawString(strokes, point, baseline);
	/* 
	 * Line 5 Nelson & Halpern numbers
	 */
	point = BORDER;
	baseline += lineHeight;
	if (nelson != null){
	    g.setFont(titleFont);
	    g.drawString(nelLabel, point, baseline);
	    point += nelLabAdvance + BORDER;
	    g.setFont(dataFont);
	    g.drawString(nelson, point, baseline);
	    point += dataMetrics.stringWidth(nelson)+ BORDER;
	}
	if (halpern != null) {
	    g.setFont(titleFont);
	    g.drawString(halLabel, point, baseline);
	    point += halLabAdvance;
	    g.setFont(dataFont);
	    g.drawString(halpern, point, baseline);
	}
	/*
	 * Line 6 and up English definitions
	 */
	if (eigo != null){
	    point = BORDER;
	    baseline += lineHeight;
	    g.setFont(titleFont);
	    g.drawString(eiLabel, point, baseline);
	    point += eiLabAdvance;
	    g.setFont (dataFont);
	    g.drawString(eigo, point, baseline);
	}
    }
}





