/*
 * This program is Copyright 1998 by Henry Cohen commercial rights are reserved.
 * The program is provided as is with no warranties of any sort including 
 * fitness for any purpose.  In no case will the author be held liable for
 * damages related to the use of this program.

 * The author hereby grants rights to use, copy, modify and create derived
 * products for individual, non-commercial purposes provided a copy of this
 * notice is included.  Any commercial use requires the written approval of 
 * the author.
 * Henry Cohen 1998.
 */

package jdic;
import java.awt.*;
import java.awt.event.*;
/**
 * This class MyTextArea is a kludge to work around bugs in java.awt.TextArea
 * In Java 1.1.4 the superclass does not compute the selection correctly when 
 * the field contains Kanji and other 2byte per character glyphs.  The width of
 * the field is not correctly computed either but I am not going to worry about that
 * right now.
 */
public class MyTextArea extends TextArea {
    int width;
    
    public MyTextArea (){
	super();
    }
    public MyTextArea(int rows, int columns){
	super(rows, columns);
    }
    public MyTextArea(String s){
	super(s);
    }
    public MyTextArea(String s, int rows, int columns){
	super(s,rows,columns);
    }
    public MyTextArea(String s, int rows, int columns, int sb){
	super(s,rows,columns,sb);
    }
    /*
     * The superclass getSelectionStart method (wrongly) returns a byte index
     * convert to a char index under the assumption that 7bit ascii is 1 byte
     * anything else is 2 bytes except half width katakana which is 1 byte.
     * This patch will only work with Japanese JIS and 7 bit ASCII.  I don't
     * know what might happen with other regions of the Unicode universe.
     * It also assumes that selections are a contiguous substring of the object's
     * text.
     */
    public int getSelectionStart() {
	int startByte = super.getSelectionStart();
	String text = super.getText();
	int nextChar = 0, nextByte = 0;
	char c;
	
	while (nextByte<startByte){
	    c = text.charAt(nextChar++);
	    if ((0x20<=c)&&(c<=0x7e)) {  // Printable ASCII Roman
		nextByte += 1;
	    } else if ((0xFF65<c)&&(c<0xff9d)){ //Halfwidth Katakana
		nextByte += 1;
	    } else { // assume full width kanji or kana
		nextByte += 2;
	    }
	}
	return nextChar;
    }
     public int getSelectionEnd(){
 	String text = super.getText();
	int endByte = super.getSelectionEnd();
	int nextChar = 0, nextByte = 0;
	char c;
	
	while (nextByte<endByte){
	    c = text.charAt(nextChar++);
	    if ((0x20<=c)&&(c<=0x7e)) {  // ASCII Roman
		nextByte += 1;
	    } else if ((0xFF65<c)&&(c<0xff9d)){ //Halfwidth Katakana
		nextByte += 1;
	    } else { // assume full width kanji or kana
		nextByte += 2;
	    }
	}
	return nextChar;
     }
     public String getSelectedText(){
	 String text = super.getText();
	 int start = this.getSelectionStart();
	 int end = this.getSelectionEnd();
	 return text.substring(start,end);
     }
}
