
/*
 * This program is Copyright 1998 by Henry Cohen commercial rights are reserved.
 * The program is provided as is with no warranties of any sort including 
 * fitness for any purpose.  In no case will the author be held liable for
 * damages related to the use of this program.

 * The author hereby grants rights to use, copy, modify and create derived
 * products for individual, non-commercial purposes provided a copy of this
 * notice is included.  Any commercial use requires the written approval of 
 * the author.
 * Henry Cohen 1998.
 */
package jdic;
import java.util.*;

/** 
 * Table of Radicals. 
 */
class RadTable {
    /*
     * The array radData contains initialization information for
     * the radical table.  It is a three dimensional array containing 
     * stroke number, radical number and unicode glyph information.
     * N.B. What I mean by glyph is not the same thing as what Java 1.2
     * means by glyph.  What I am referring to is the particular form of a 
     * traditional radical (bushu).  Unicode has codes for most of these but 
     * not for all variants.  Sometimes we will need to use a kanji that uses
     * the particular form but has additional strokes.  In these cases I will
     * darken part of the cell in which the character is draw to try to highlight
     * the actual radical part.  This is all just hueristic anyway.  There is not
     * sufficient information in kanjidic at present to do unambiguous searches
     * by radical and stroke.  What is needed is a unique code for each variant
     * of the radical.  That way if a particular radical has different variants
     * with different stroke counts we could still compute the sum unambiguously.
     *
     * [Soap] I would really like to have an array of structures but Java doesn't 
     * let me do that.  Having radical number and glyph in the same array is
     * really kludgy but the alternative seems to be to have multiple arrays with
     * identical structure one for each component of the record (Anybody remember
     * FORTRAN?).  That would be errorprone because the arrays must have exactly
     * the same shape.
     */
    static char[][][] radData = {
	{/* 1 Stroke */
	    {1,'\u4e00'},
	    {2,'\u4e28'},
	    {3,'\u4e36'},
	    {4,'\u30ce','\u4e3f'},
	    {5,'\u4e59','\u4e5a'},
	    {6,'\u4e85'}
	},
	{/* 2 Strokes */
	    {7,'\u4e8c'},
	    {8,'\u4ea0'},
	    {9,'\u4eba','\u4ebb','\u4ebc'},
	    {10,'\u513f'},
	    {11,'\u5165'},
	    {12,'\u516b'},
	    {13,'\u5182'},
	    {14,'\u5196'},
	    {15,'\u51ab'},
	    {16,'\u51e0'},
	    {17,'\u51f5'},
	    {18,'\u5200','\u5202'},
	    {19,'\u529b'},
	    {20,'\u52f9'},
	    {21,'\u5315'},
	    {22,'\u531a'},
	    {23,'\u5338'},
	    {24,'\u5341'},
	    {25,'\u535c'},
	    {26,'\u5369'},
	    {27,'\u5382'},
	    {28,'\u53b6'},
	    {29,'\u53c8'},
	    {162,'\u8fb6'},
	},
	{/* 3 Strokes */
	    {30,'\u53e3'},
	    {31,'\u56d7'},
	    {32,'\u571f'},
	    {33,'\u58eb'},
	    {34,'\u5902'},
	    {35,'\u590a'},
	    {36,'\u5915'},
	    {37,'\u5927'},
	    {38,'\u5973'},
	    {39,'\u5b50'},
	    {40,'\u5b80'},
	    {41,'\u5bf8'},
	    {42,'\u5c0f'},
	    {43,'\u5c22'},
	    {44,'\u5c38'},
	    {45,'\u5c6e'},
	    {46,'\u5c71'},
	    {47,'\u5ddd','\u5ddb'},
	    {48,'\u5de5'},
	    {49,'\u5df1','\u5df2','\u5df3'},
	    {50,'\u5dfe'},
	    {51,'\u5e72'},
	    {52,'\u5e7a'},
	    {53,'\u5e7f'},
	    {54,'\u5ef4'},
	    {55,'\u5efe'},
	    {56,'\u5f0b'},
	    {57,'\u5f13'},
	    {58,'\u30e8','\u5f51'},
	    {59,'\u5f61'},
	    {60,'\u5f73'},
	    {61,'\u5fc4'},
	    {64,'\u624c'},
	    {85,'\u6c35'},
	    {90,'\u58ef'},   // If a real Unicode font is used this will be wrong.
	    {94,'\u72ad'},
	    {140,'\u8279'},
	    {162,'\u8fb5'},
	    {163,'\u9092'},
	    {170,'\u961d'},
	},
	{/* 4 Strokes */
	    {61,'\u5fc3'},
	    {62,'\u6208'},
	    {63,'\u6238'},
	    {64,'\u624b'},
	    {65,'\u652f'},
	    {66,'\u6534','\u6535'},
	    {67,'\u6587'},
	    {68,'\u6597'},
	    {69,'\u65a4'},
	    {70,'\u65b9'},
	    {71,'\u65e0','\u65e1'},
	    {72,'\u65e5'},
	    {73,'\u66f0'},
	    {74,'\u6708'},
	    {75,'\u6728'},
	    {76,'\u6b20'},
	    {77,'\u6b62'},
	    {78,'\u6b79'},
	    {79,'\u6bb3'},
	    {80,'\u6bcb'},
	    {81,'\u6bd4'},
	    {82,'\u6bdb'},
	    {83,'\u6c0f'},
	    {84,'\u6c14'},
	    {85,'\u6c34'},
	    {86,'\u706b','\u706c'},
	    {87,'\u722a','\u722b'},
	    {88,'\u7236'},
	    {89,'\u723b'},
	    {90,'\u723f'},
	    {91,'\u7247'},
	    {92,'\u7259'},
	    {93,'\u725b'},
	    {94,'\u72ac'},
	    {96,'\u738b'},
	    {113,'\u30cd'},
	    {125,'\u8002'},
	    {130,'\u6708'},
	    {140,'\u8279'}
	},
	{/* 5 Strokes */
	    {80,'\u6bcd'},
	    {85,'\u6c3a'},
	    {95,'\u7384'},
	    {96,'\u7389'},
	    {97,'\u74dc'},
	    {98,'\u74e6'},
	    {99,'\u7518'},
	    {100,'\u751f'},
	    {101,'\u7528'},
	    {102,'\u7530'},
	    {103,'\u758b'},
	    {104,'\u7592'},
	    {105,'\u7676'},
	    {106,'\u767d'},
	    {107,'\u76ae'},
	    {108,'\u76bf'},
	    {109,'\u76ee'},
	    {110,'\u77db'},
	    {111,'\u77e2'},
	    {112,'\u77f3'},
	    {113,'\u793a'},
	    {114,'\u79b8'},
	    {115,'\u79be'},
	    {116,'\u7a74'},
	    {117,'\u7acb'},
	    {122,'\u7f52'},
	    {145,'\u8864'},
	},
	{/* 6 Strokes */
	    {118,'\u7af9'},
	    {119,'\u7c73'},
	    {120,'\u7cf8'},
	    {121,'\u7f36'},
	    {122,'\u7f51'},
	    {123,'\u7f8a'},
	    {124,'\u7fbd'},
	    {125,'\u8001'},
	    {126,'\u800c'},
	    {126,'\u8012'},
	    {128,'\u8033'},
	    {129,'\u807f'},
	    {130,'\u8089'},
	    {131,'\u81e3'},
	    {132,'\u81ea'},
	    {133,'\u81f3'},
	    {134,'\u81fc'},
	    {135,'\u820c'},
	    {136,'\u821b'},
	    {137,'\u821f'},
	    {138,'\u826e'},
	    {139,'\u8272'},
	    {140,'\u8278'},
	    {141,'\u864d'},
	    {142,'\u866b'},
	    {143,'\u8840'},
	    {144,'\u884c'},
	    {145,'\u8863'},
	    {146,'\u897f','\u897e'},
	},
	{/* 7 Strokes */
	    {131,'\u81e3'},
	    {136,'\u821b'},
	    {147,'\u898b'},
	    {148,'\u89d2'},
	    {149,'\u8a00'},
	    {150,'\u8c37'},
	    {151,'\u8c46'},
	    {152,'\u8c55'},
	    {153,'\u8c78'},
	    {154,'\u8c9d'},
	    {155,'\u8d64'},
	    {156,'\u8d70'},
	    {157,'\u8db3'},
	    {158,'\u8eab'},
	    {159,'\u8eca'},
	    {160,'\u8f9b'},
	    {161,'\u8fb0'},
	    {163,'\u9091'},
	    {164,'\u9249'},
	    {165,'\u91c6'},
	    {166,'\u91cc'},
	    {168,'\u9578'},
	    {199,'\u9ea6'},
	},
	{/* 8 Strokes */
	    {167,'\u91d1'},
	    {168,'\u9577'},
	    {169,'\u9580'},
	    {170,'\u961c'},
	    {171,'\u96b6'},
	    {172,'\u96b9'},
	    {173,'\u96e8'},
	    {174,'\u9752','\u9751'},
	    {175,'\u975e'},
	    {184,'\u98e0'},
	    {210,'\u6589'},
	},
	{/* 9 Strokes */
	    {176,'\u9762'},
	    {177,'\u9769'},
	    {178,'\u97cb'},
	    {179,'\u97ed'},
	    {180,'\u97f3'},
	    {181,'\u9801'},
	    {182,'\u98a8'},
	    {183,'\u98db'},
	    {184,'\u98df'},
	    {185,'\u9996'},
	    {186,'\u9999'},
	},
	{/* 10 Strokes */
	    {187,'\u99ac'},
	    {188,'\u9aa8'},
	    {189,'\u9ad8'},
	    {190,'\u9adf'},
	    {191,'\u9b25'},
	    {192,'\u9b2f'},
	    {193,'\u9b32'},
	    {194,'\u9b3c'},
	    {212,'\u7adc'},
	},
	{/* 11 Strokes */
	    {189,'\u9ad9'},
	    {195,'\u9b5a'},
	    {196,'\u9ce5'},
	    {197,'\u9e75'},
	    {198,'\u9e7f'},
	    {199,'\u9ea5'},
	    {200,'\u9ebb'},
	    {201,'\u9ec4'},
	    {203,'\u9ed2'},
	    {213,'\u4e80'},
	},
	{/* 12 Strokes */
	    {202,'\u9ecd'},
	    {203,'\u9ed1'},
	    {204,'\u9ef9'},
	    {211,'\u6b6f'},
	},
	{/* 13 Strokes */
	    {205,'\u9efd'},
	    {206,'\u9f0e'},
	    {207,'\u9f13'},
	    {208,'\u9f20'},
	},
	{/* 14 Strokes */
	    {209,'\u9f3b'},
	    {210,'\u9f4a'},
	},
	{/* 15 Strokes */
	    {211,'\u9f52'},
	},
	{/* 16 Strokes */
	    {212,'\u9f8d'},
	    {213,'\u9f9c'},
	},
	{/* 17 Strokes */
	    {214,'\u9fa0'},
	},
    };
    /*
     * Table of Unicode radicals with no JIS equivalent.
     * Format is 2xN array of pairs (Desired, Substitute)  The substitute
     * is a character with a JIS mapping that can be used to display the
     * radical. The third entry in each row is an indication of where in
     * the character square the radical is to be found.  The comment /*Fake* /
     * in this table means that there isn't even a Unicode code for the radical
     * form so I have chosen something else to substitute that also does not
     * have a JIS encoding.  My hope is that I can detect these cases and use
     * the substitute. The table is sorted by the first entry in each row.
     */
    static final char ALL = 1;
    static final char ENCLOSURE = 3;
    static final char LEFT = 4;
    static final char RIGHT = 5;
    static final char TOP = 6;
    static final char BOTTOM = 7;
    /*
     * Mapping table for Unicode characters with no corresponding JIS.
     * One row per map entry.  
     * First char is original Unicode, second is a substitute Unicode with a JIS
     * mapping and what follows is information for drawing highlghted areas to 
     * indicate what part of the character is the radical form.  The drawing 
     * information consists of a constant to indicate whether to draw the dark
     * area first or the light area first, if they overlap it can make a difference.
     * Then two RealRect objects contain coordinates for rectangles to be 
     * respectively lightened and darkened to hilight the radical part.  The first 
     * rectangle is the highlighted area and the second the darkened area.  
     * Rectangles are defined by their upper-left and lower-right corners.  The
     * coordinate space is the same as java.awt Origin at upper left x increasing
     * to the right and y increasing downward.  The dimensions are actually scale
     * factors that will be applied to a character drawing cell so they will always
     * be fractions between 0 and 1. 
     * For example if a radical is on top and takes up 1/2 of the cell its
     * dimensions will be Light ((0,0),(1,1/2)) Dark ((0,1/2)(1,1)).
     */ 
    static Substitute[] noJIS = {
	new Substitute ('\u4e5a','\u4e73',
			new RealRect(0.5f, 0.0f, 1.0f, 1.0f),
			new RealRect(0.0f, 0.0f, .5f, 1.0f),
			Substitute.DARK_LIGHT),
	new Substitute ('\u4ebb','\u4ec6',
			new RealRect (0.f, 0.f, .5f, 1.f),
			new RealRect (.5f, 0.f, 1.f, 1.f),
			Substitute.DARK_LIGHT),
	new Substitute ('\u4ebc','\u4e2a',
			new RealRect (0.f, 0.f, 1.f, 2.f/5.f),
			new RealRect (0.f, 2.f/5.f, 1.f, 1.f),
			Substitute.DARK_LIGHT),
	new Substitute ('\u5202','\u5208',
			new RealRect (.5f, 0.f, 1.0f, 1.0f),
			new RealRect (0.0f, 0.0f, .5f, 1.0f),
			Substitute.DARK_LIGHT),
	new Substitute ('\u58ef'/*Fake*/,'\u58ee',
			new RealRect (0.f, 0.f, 2.f/5.f, 1.f),
			new RealRect (2.f/5.f, 0.f, 1.f, 1.f),
			Substitute.DARK_LIGHT),
	new Substitute ('\u5fc4','\u5fd6',
			new RealRect (0.0f, 0.0f, .5f, 1.f),
			new RealRect (1.f/5.f, 0.f, 1.f, 1.f),
			Substitute.DARK_LIGHT),
	new Substitute ('\u624c','\u6253',
			new RealRect (0.f, 0.f, 3.f/5.f, 1.f),
			new RealRect (3.f/5.f, 0.f, 1.f, 1.f),
			Substitute.DARK_LIGHT),
	new Substitute ('\u6c35','\u6c41',
			new RealRect (0.f, 0.f, 12.f/25.f, 1.f),
			new RealRect (12.f/25.f, 0.f, 1.f, 1.f),
			Substitute.DARK_LIGHT),
	new Substitute ('\u6c3a','\u6cf0',
			new RealRect (1.f/5.f, .5f, 4.f/5.f, 1.f),
			new RealRect (0.f, 0.f, 1.f, 1.f),
			Substitute.DARK_LIGHT),
	new Substitute ('\u706c','\u70cb',
			new RealRect (0.f, .75f, 1.f, 1.f),
			new RealRect (0.f, 0.f, 1.f, .75f),
			Substitute.DARK_LIGHT),
	new Substitute ('\u722b','\u59a5',
			new RealRect (0.f, 0.f, 1.f, .5f),
			new RealRect (0.f, .5f, 1.f, 1.f),
			Substitute.DARK_LIGHT),
	new Substitute ('\u72ad','\u72b2',
			new RealRect (0.f, 0.f, .5f, 1.f),
			new RealRect (.5f, 0.f, 1.f, 1.f),
			Substitute.DARK_LIGHT),
	new Substitute ('\u7592','\u7594',
			new RealRect (0.f, 0.f, 1.f, 1.f),
			new RealRect (2.f/5.f, 8.f/25.f, 1.f, 1.f),
			Substitute.LIGHT_DARK),
	new Substitute ('\u79b8','\u79b9',
			new RealRect (0.f, .5f, 1.f, 1.f),
			new RealRect (0.f, 0.f, 1.f, 4.f/10.f),
			Substitute.DARK_LIGHT),
	new Substitute ('\u7f52','\u7f58',
			new RealRect (0.f, 0.f, 1.f, 2.f/5.f),
			new RealRect (0.f, 2.f/5.f, 1.f, 1.f),
			Substitute.DARK_LIGHT),
	new Substitute ('\u8002','\u8001',
			new RealRect (0.f, 0.f, 1.f, 1.f),
			new RealRect (2.f/5.f, 3.f/5.f, 1.f, 1.f),
			Substitute.LIGHT_DARK),
	new Substitute ('\u8279','\u827e',
			new RealRect (0.f, 0.f, 1.f, 2.f/5.f),
			new RealRect (0.f, 2.f/5.f, 1.f, 1.f),
			Substitute.DARK_LIGHT),
	new Substitute ('\u8864','\u886b',
			new RealRect (0.f, 0.f, .5f, 1.f),
			new RealRect (.5f, 0.f, 1.f, 1.f),
			Substitute.DARK_LIGHT),
	new Substitute ('\u8fb5','\u8fb7',
			new RealRect (0.f, 0.f, 1.f, 1.f),
			new RealRect (2.f/5.f, 0.f, 1.f, 3.f/5.f),
			Substitute.LIGHT_DARK),
	new Substitute ('\u8fb6','\u8fbb',
			new RealRect (0.f, 0.f, 1.f, 1.f),
			new RealRect (2.f/5.f, 0.f, 1.f, 3.f/5.f),
			Substitute.LIGHT_DARK),
	new Substitute ('\u9092','\u90a6',
			new RealRect (3.f/5.f, 0.f, 1.f, 1.f),
			new RealRect (0.f, 0.f, .5f, 1.f),
			Substitute.DARK_LIGHT),
	new Substitute ('\u9578','\u8086',
			new RealRect (0.f, 0.f, .5f, 1.f),
			new RealRect (.5f, 0.f, 1.f, 1.f),
			Substitute.DARK_LIGHT),
	new Substitute ('\u961d','\u9621',
			new RealRect (0.f, 0.f, 2.f/5.f, 1.f),
			new RealRect (2.f/5.f, 0.f, 1.f, 1.f),
			Substitute.DARK_LIGHT),
	new Substitute ('\u9751','\u9759', 
			new RealRect (0.f, 0.f, .5f, 1.f),
			new RealRect (.5f, 0.f, 1.f, 1.f),
			Substitute.DARK_LIGHT),
	new Substitute ('\u98e0','\u98f2',
			new RealRect (0.f, 0.f, .5f, 1.f),
			new RealRect (.5f, 0.f, 1.f, 1.f),
			Substitute.DARK_LIGHT),
    };
    /*
     * Inner class Radical.  This is the cell type for the Radical Table.
     */
    class Radical {
	short radicalNumber;
	byte strokes;
	char glyph;
	boolean pureRadical;
	byte position;
	Radical (char radicalNumber, int strokes, char glyph) {
	    this.radicalNumber = (short)radicalNumber;
	    this.strokes = (byte)strokes;
	    this.glyph = glyph;
	}
    }
    /*
     * The following is a static initializer block to set up the radical table.
     * Static in the Java sense of a class initializer, it is actually executed
     * at runtime when the class is loaded.
     * The table is a three dimensional array radTable[stroke][radical][glyph].
     * Most radicals have only one glyph but some have multiples.  
     * The stroke index is the actual stroke count minus 1 because of zero offset
     * array addressing.  The radical index is the ordinal position of the radical
     * within the stroke count.  The actual radical number is stored in each 
     * Radical object. If the 
     * alternative forms have different numbers of strokes then there will be
     * multiple rows for the same radical at the different stroke counts.
     * [Soap] If Java had structures that I could initialize statically then
     * I would not need to go through all of this.  I find the following fairly
     * incomprehensible myself so don't expect me to explain.  Please refer to
     * Section  15.9.1 of the Java 1.0 Language Definition document for an 
     * explaination.  Anyway, it works.
     */
    static Radical radTable[][][] = new Radical[17][][];
    {
	int radCount;
	for (int strokes = 0; strokes<radTable.length; strokes++) {
	    radCount = radData[strokes].length;
	    radTable[strokes] = new Radical[radCount][];
	    for (int radical = 0; radical<radCount; radical++) {
		int glyphCount = radData[strokes][radical].length-1;
		radTable[strokes][radical] = new Radical[glyphCount];
		for (int glyph = 1; glyph<glyphCount+1; glyph++){
		    radTable[strokes][radical][glyph-1] = 
			new Radical(radData[strokes][radical][0], strokes+1,
				    radData[strokes][radical][glyph]);
		}
	    }
	}
    }
    /*
     * The constructor for the RadTable class doesn't have to do anything
     * since the class is mostly static.  The constructor is necessary to
     * force the static initialization code to run.
     * I include one so that I can force the class to be loaded before it is needed.
     */
    public RadTable(){
    }
    /*
     * method glyphList(strokes) given a stroke count return a list of unicode
     * characters representing radicals with that number of strokes.  Some of these
     * characters may not have representations in the available fonts.
     * The list is an array of strings because that is what the KanjiPanel object
     * wants.
     */ 
    static String[] glyphList(short strokes) {
	char[] carray = new char[1];
	Vector tmp = new Vector();
	int radCount = radTable[strokes-1].length;
	
	for(byte radical = 0; radical < radCount; radical++){
	    int glyphCount = radTable[strokes-1][radical].length;
	    for(byte glyph = 0; glyph < glyphCount; glyph++) {
		carray[0] = radTable[strokes-1][radical][glyph].glyph;
		tmp.addElement(new String(carray));
	    }
	}
	String[] list = new String[tmp.size()];
	tmp.copyInto(list);
	return list;
    }
    /*
     * method radNumber(short strokes, char glyph) given a stroke count and a 
     * glyph return the bushu number.  The glyph must be in the table.  I assume
     * that this method will always be called with a glyph that was returned by
     * a previous call to glyphList().
     */
    static short radNumber(short strokes, char glyph){
	Radical[][] strokeTab = radTable[strokes-1];
	short radical;
	int radCount = strokeTab.length;
	for (byte radRec = 0; radRec < radCount; radRec++){
	    Radical[] glyphTab = strokeTab[radRec];
	    for(byte radGlyph = 0; radGlyph < glyphTab.length; radGlyph++){
		if(glyphTab[radGlyph].glyph == glyph) 
		    return glyphTab[radGlyph].radicalNumber;
	    }
	}
	/*
	 * Should never happen!
	 */
	return 0;
    }
    public static  void main(String[] args){
	RadTable rt = new RadTable();
	
	for (int strokes = 0; strokes< radTable.length;strokes++) {
	    int radicals = radTable[strokes].length;
	    int sg = 0;

	    for (int rad = 0; rad<radicals; rad++){
		int glyphs = radTable[strokes][rad].length;
		sg += glyphs;
	    }
	}
    }
    /*
     * If there is no glyph in the available fonts for a particular 
     * Unicode character then propose a substitute.  The substitute
     * is a kanji that uses the same radical but may have more strokes.
     * The substitute also tells where the radical is so that clients
     * can hilight the radical part.  Hilighting is just a hueristic,
     * it may look terrible.  The best solution would be to have a 
     * complete Unicode CJK Unification font but even that has some 
     * radicals missing.

     * The algorithm is simple linear search.  If the list was longer
     * I would use binary search.
     */
    static Substitute radSubstitute(char original) {
	Substitute result = null;
		
        loop: for (int i = 0; i< noJIS.length; i++){
	    if (original == noJIS[i].original){
		result = noJIS[i];
		break loop;
	    } else if (original < noJIS[i].original) break loop;
	}
	return result;
    }
}



