/*
 * This program is Copyright 1998 by Henry Cohen commercial rights are reserved.
 * The program is provided as is with no warranties of any sort including 
 * fitness for any purpose.  In no case will the author be held liable for
 * damages related to the use of this program.

 * The author hereby grants rights to use, copy, modify and create derived
 * products for individual, non-commercial purposes provided a copy of this
 * notice is included.  Any commercial use requires the written approval of 
 * the author.
 * Henry Cohen 1998.
 */

/**
 * Class searchList - class to implement ordered list insertion and searching.
 * methods include insert, search and delete.
 * I plan to try at least two algorithms; AVL balanced trees because I like them
 * and hash tables because they are fast.  I want to see if reasonable sized 
 * hash tables are really faster than balanced tree searching.  AVL trees should
 * work well because we don't do a lot of insertion and no deletion at all
 * but we will do a lot of searching. 
 * For references on AVL trees see Knuth vol3. 
 * This algorithm is from Fundamentals lf Data Structures by Horowitz and Sahni
 * I don't like all of the temporary variables but it is the reference at hand.
 * The other reason for doing AVL trees
 * rather than hash tables is that it gives me some experience doing linked
 * data structures in Java.   
 */  
package jdic;
public class SearchList {  // AVL Balanced Tree implementation
    String key;
    SearchList left;
    SearchList right;
    byte balance;
      
    public SearchList(String key){
	this.key = key;
	this.balance = 0;
	this.left = null;
	this.right = null;
    }
    /**
     * method insert(key) insert key into tree
     * For now use plain binary trees.  Balance them later but get this
     * working now.
     */
    public  void insert(String key){
	int order = key.compareTo(this.key);

	
	if (order < 0){
	    if (left != null) {
		left.insert(key);
	    } else {
		left = new SearchList(key);
	    }
	} else if (order > 0) {
	    if (right != null) {
		right.insert(key);
	    } else {
		right = new SearchList(key);
	    }
	} else {
	    return;
	}
    }
    public void delete(String key){
	/*
	 * Unimplemented.  Deletion is a bit too hard for this application
	 * also unnecessary for this application.
	 */
    }
    public boolean contains(String key){
	int order = this.key.compareTo(key);
	if (order < 0){
	    if (left != null){
		return left.contains(key);
	    } else {
		return false;
	    }
	} else if (order > 0){
	    if (right != null) {
		return right.contains(key);
	    } else {
		return false;
	    }
	} else if (order == 0){
	    return true;
	}
	/*
	 * Another not reached return!
	 */
	return true;
    }
    /*
     * Inorder traversal so I can check that the tree is properly constructed.
     * should list all elements in lexicographic order
     */
    public void inorder() {
	if (left != null) left.inorder();
	System.err.print(key + " ");
	if (right != null) right.inorder();
    }
    public void preorder(){
	System.err.print(key + " ");
	if (left != null) left.inorder();
	if (right != null) right.inorder();
    }
}

